<?php

/*
Plugin Name: Splinder Importer
Plugin URI: http://www.aioros.net/
Description: Import posts, comments and categories from a Splinder blog.
Author: Aioros
Author URI: http://www.aioros.net/
Version: 0.1
Stable tag: 0.1
License: GPL v3 - http://www.gnu.org/licenses/gpl-3.0.html
*/

if ( !defined('WP_LOAD_IMPORTERS') )
	return;

// Load Importer API
require_once ABSPATH . 'wp-admin/includes/import.php';

if ( !class_exists( 'WP_Importer' ) ) {
	$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
	if ( file_exists( $class_wp_importer ) )
		require_once $class_wp_importer;
}

/**
 * Splinder Importer
 *
 * @package WordPress
 * @subpackage Importer
 */
if ( class_exists( 'WP_Importer' ) ) {

class Splinder_Import extends WP_Importer {

	var $importer_code;
	var $importer_name;
	var $importer_desc;
	var $today;
	var $totalposts;
	var $totalcomments;
	
	function Splinder_Import() {
		$this->importer_code = "splinderimporter";
		$this->importer_name = "Splinder Importer";
		$this->importer_desc = "Import from Splinder";
		$this->today = getdate();
		$this->totalposts = 0;
		$this->totalcomments = 0;
	}

	function header() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>'.__('Splinder Import', 'splinder-importer').'</h2>';
	}

	function footer() {
		echo '</div>';
	}

	function greet() {
		echo '<div class="narrow">';
		echo '<p>'.__('Prepara il tuo template, poi scegli un blog Splinder, una data di inizio e una di fine.', 'splinder-importer').'</p>';
		$this->import_splinder_form("admin.php?import=" . $this->importer_code . "&amp;step=1");
		echo '</div>';
	}

	// import links?
	
	function import($blogurl, $start_month, $start_year, $end_month, $end_year) {
		$month = $start_month;
		$year = $start_year;
		while ( ($year < $end_year) || (($year == $end_year) && ($month <= $end_month)) ) {
	
			echo sprintf("%04d-%02d", $year, $month) . "<br />";
			$url = $blogurl . "/archive/" . sprintf("%04d-%02d", $year, $month);
			$html = file_get_contents($url) or die("Could not access file: $url");

			$inputposts = explode("\$\$\$POSTSEPARATOR\$\$\$", $html);
			array_splice($inputposts, 0, 1);
			array_splice($inputposts, -1, 1);
			$inputposts = array_reverse($inputposts);
			$this->totalposts = $this->totalposts + count($inputposts);
			echo "Posts: " . count($inputposts) . "<br /><br />";

			foreach ($inputposts as $post) {
				list( , $number, ) = explode("\$\$\$ITEMNUMBER\$\$\$", $post);
				$postpage = file_get_contents($blogurl . "/post/" . $number . "#comment");
				list( , $datetime, ) = explode("\$\$\$ITEMDATETIME\$\$\$", $postpage);
				preg_match("#([0-9]{2})/([0-9]{2})/([0-9]{2,4}) ([0-9]{2}:[0-9]{2})#", $datetime, $datetime);
				$datetime = "$datetime[3]-$datetime[2]-$datetime[1] $datetime[4]:00";
				list( , $categories, ) = explode("\$\$\$ITEMCATS\$\$\$", $postpage);
				list( , $title, ) = explode("\$\$\$ITEMTITLE\$\$\$", $postpage);
				preg_match("#<a href=.+>(.+)</a>#", $title, $title);
				$title = $title[1];
				list( , $body, ) = explode("\$\$\$ITEMBODY\$\$\$", $postpage);
				$body = substr($body, 0, strpos($body, "<!--"));
				list( , $author, ) = explode("\$\$\$ITEMAUTHOR\$\$\$", $postpage);
				list( , $comments, ) = explode("\$\$\$COMMENTS\$\$\$", $postpage);
				$comDates = $this->splinder_com_dates($comments);
				$comBodies = $this->splinder_com_texts($comments);
				$comAuthors = $this->splinder_com_authors($comments);

				// insert author
				$author_id = $this->add_user($author);

				// insert categories and tags
				$cat_ids = array();
				$tag_ids = array();
				$tag_names = array();
				
				if ($_POST['tagsorcats']=="tags") {
					$ntags = preg_match_all("#tag'>([^<]+)</a>#Us", $categories, $cleantags);
					if ($ntags > 0) {
						for ($i = 0; $i < count($cleantags[0]); $i++) {
							$tag_ids[$i] = $this->add_tag($cleantags[1][$i]);
							$tag_ids[$i] = $tag_ids[$i]['term_id'];
							$tag_names[$i] = $cleantags[1][$i];
						}
					}
					$tag_names = implode(",", $tag_names);
					if ($_POST['uncategorized']=="default")
						$cat_ids = array(get_option('default_category'));
					else if ($_POST['uncategorized']=="other")
						$cat_ids = array($this->add_category($_POST['other-category']));
				}
				
				if ($_POST['tagsorcats']=="cats") {
					$ncats = preg_match_all("#tag'>([^<]+)</a>#Us", $categories, $cleancats);
					if ($ncats > 0) {
						for ($i = 0; $i < count($cleancats[0]); $i++) {
							$cat_ids[$i] = $this->add_category($cleancats[1][$i]);
						}
					} else {
						if ($_POST['uncategorized']=="default")
							$cat_ids = array(get_option('default_category'));
						else if ($_POST['uncategorized']=="other")
							$cat_ids = array($this->add_category($_POST['other-category']));
					}
				}
				
				// insert post
				$post_id = $this->add_post($title, $body, $author_id, $datetime, $cat_ids, $tag_names);

				// insert comments
				for ($i = 0; $i < count($comDates[0]); $i++) {
					$comDate = $comDates[3][$i] . "-" . $this->month_numberize($comDates[2][$i]) . "-" . $comDates[1][$i] . " " . $comDates[4][$i] . ":00";
					$comment_id = $this->add_comment($post_id, $comBodies[1][$i], $comAuthors[3][$i], $comAuthors[2][$i], $comDate);
				}
				$this->totalcomments = $this->totalcomments + count($comDates[0]);
					
			}
	
			$month = $month % 12 + 1;	// - what, that actually works? - i know, right?
			$year = $year + ($month == 1);
	
		}

		echo "<br />Total posts: " . $this->totalposts;
		echo '<h3>'.sprintf(__('All done. <a href="%s">Have fun!</a>'), get_option('home')).'</h3>';
	}

	// code from Terenzani's plugin (http://www.terenzani.it/38/importare-i-post-di-splinder-in-wordpress/)
	// Example: $comDates[i][j] == i-th datetime element (from regex capturing group) of the j-th comment

	function splinder_com_dates($comments) {
		preg_match_all("#:right\">([0-9]{1,2}) ([\w]+) ([0-9]{4}) - ([0-9]{2}:[0-9]{2})#", $comments, $comDates);
		if (!empty($comDates)) return $comDates;
		return false;
	}
	function splinder_com_texts($comments) {
		preg_match_all("#<div class=\"comment-body\">(.+)</div>#Us", $comments, $comBodies);
		if (!empty($comBodies)) return $comBodies;
		return false;
	}
	function splinder_com_authors($comments) {
		preg_match_all("#<td class=\"icons\">[\r\n\t]+(<a href=\"(.+)\".+)?</td>[\r\n\t]+<td>[\r\n\t]+([\w\d \._-]+)[\r\n\t]+</td>#Us", $comments, $comAuthors);
		if (empty($comAuthors[0])){
			preg_match_all("#<div class=\"comment-bottom\">[\r\n\t]+([\w\d \._-]+)[\r\n\t]+.+(<a href=\"(http://(?!www\.motime\.com).+)\"|<br />)#Us", $comments, $comAuthors);
			$comAuthors[2] = $comAuthors[3];
			$comAuthors[3] = $comAuthors[1];
		}
		if (!empty($comAuthors)) return $comAuthors;
		return false;
	}
	function month_numberize($string){
		$a1 = array("gennaio", "febbraio", "marzo", "aprile", "maggio", "giugno", "luglio", "agosto", "settembre", "ottobre", "novembre", "dicembre");
		$a3 = array('january', 'february', 'mars', 'april', 'may', 'june', 'july', 'august', 'september', 'october', 'november', 'december');
		$a2 = array(1,2,3,4,5,6,7,8,9,10,11,12);
		$string = str_replace($a1,$a2,strtolower ($string));
		return $string = str_replace($a3,$a2,strtolower ($string));
	}
	function month_tostring($month, $lang='it'){
		$a1 = array("Ottobre", "Novembre", "Dicembre", "Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno", "Luglio", "Agosto", "Settembre");
		$a3 = array('October', 'November', 'December', 'January', 'February', 'Mars', 'April', 'May', 'June', 'July', 'August', 'September');
		$a2 = array(10,11,12,1,2,3,4,5,6,7,8,9);
		if ($lang=='it') return str_replace($a2,$a1,$month);
		return str_replace($a2,$a3,$month);
	}

	// code from IoBloggo importer (http://www.andreabeggi.net/2008/10/30/importare-da-iobloggo-a-wordpress/)
	function add_user($username) {
		$user_id = username_exists($username);
		if (!$user_id) {
			$admin_info = get_userdata(1);
			if ($username == $admin_info->nickname) return 1;
			$pass = substr(md5(uniqid(microtime())), 0, 6);
			$user_id = wp_create_user($username, $pass);
			echo "Creato utente: $username<br />";
		}
		return $user_id;
	}
	function add_category($catname) {
		$category_id = category_exists($catname);
		if (!$category_id) {
			$category_id = wp_create_category($catname);
			//echo "Creata categoria: $catname<br />";
		}
		return $category_id;
	}
	function add_tag($tagname) {
		$tag_id = tag_exists($tagname);
		if (!$tag_id) {
			$tag_id = wp_create_tag($tagname);
			//echo "Creato tag: $tagname<br />";
		}
		return $tag_id;
	}
	function add_post($title, $content, $author, $date, $categories, $tags) {
		$post_id = $this->post_exists($author, $date);
		if (!$post_id) {
			$post = array();
			$post['post_title'] = $title;
			$post['post_content'] = $content;
			$post['post_author'] = $author;
			$post['post_date'] = $date;
			$post['post_category'] = $categories;
			$post['tags_input'] = $tags;
			$post['post_status'] = "publish";
			$post_id = wp_insert_post($post);
			if ($_POST['verbose-posts']=="posts") echo "Inserito post: $title<br />";
		}
		return $post_id;
	}
	function add_comment($post_id, $content, $author, $author_url, $date) {
		$comment_id = $this->comment_exists($author, $date);
		if (!$comment_id) {
			$comment = array();
			$comment['comment_post_ID'] = $post_id;
			$comment['comment_content'] = $content;
			$comment['comment_author'] = $author;
			$comment['comment_author_url'] = $author_url;
			$comment['comment_date'] = $date;
			$comment = wp_filter_comment($comment);
			$comment_id = wp_insert_comment($comment);
			if ($_POST['verbose-comments']=="comments") echo "Inserito commento di $author in data $date<br />";
		}
		return $comment_id;
	}
	
	function post_exists($author, $date) {
		global $wpdb;
		$query = $wpdb->prepare("
					SELECT * FROM wp_posts
					WHERE post_author = %d
					AND post_date = %s",
					$author, $date);
		$result = $wpdb->get_results($query);
		if (count($result)>0) {
			$result = $result[0];
			if ($_POST['verbose-posts']=="posts") echo utf8_encode("Post di " . get_userdata($author)->user_login . " in data $date gi presente<br />");
			return $result->ID;
		}
		return FALSE;
	}
	function comment_exists($author, $date) {
		global $wpdb;
		$query = $wpdb->prepare("
					SELECT * FROM wp_comments
					WHERE comment_author = %d
					AND comment_date = %s",
					$author, $date);
		$result = $wpdb->get_results($query);
		if (count($result)>0) {
			$result = $result[0];
			return $result->comment_ID;
		}
		return FALSE;
	}
	
	function import_splinder_form($action) {
		?>
		<form enctype="multipart/form-data" id="import-form" method="post" action="<?php echo esc_attr(wp_nonce_url($action, 'import-splinder-form')); ?>">
		<p>
		<label for="template"><?php _e('Sostituisci il template del tuo blog con questo codice: ' ); ?></label>
		<textarea id="template" name="template" cols="80" rows="8" readonly="readonly">&lt;html&gt;&lt;body&gt;Blog in fase di aggiornamento...&lt;BR&gt;&lt;BR&gt;&lt;BR&gt;&lt;BR&gt;&lt;BR&gt;
EXPORTING BLOG...&lt;BR&gt;
&lt;Blogger&gt;$$$POSTSEPARATOR$$$
$$$ITEMNUMBER$$$&lt;$BlogItemNumber$&gt;$$$ITEMNUMBER$$$
$$$ITEMDATETIME$$$&lt;$BlogItemDateTime$&gt;$$$ITEMDATETIME$$$
$$$ITEMCATS$$$&lt;$BlogItemCategories$&gt;$$$ITEMCATS$$$
$$$ITEMTITLE$$$&lt;$BlogItemTitle$&gt;$$$ITEMTITLE$$$
$$$ITEMBODY$$$&lt;$BlogItemBody$&gt;$$$ITEMBODY$$$
$$$ITEMAUTHOR$$$&lt;$BlogItemAuthor$&gt;$$$ITEMAUTHOR$$$
$$$COMMENTS$$$&lt;$BlogItemCommentList$&gt;$$$COMMENTS$$$
&lt;/Blogger&gt;$$$POSTSEPARATOR$$$
&lt;$BlogLinkOut$&gt;
&lt;/body&gt;&lt;/html&gt;</textarea>
		</p><p>
		Dal tuo pannello di controllo di Splinder, disattiva la paginazione (Blog -&gt; Configura -&gt; Impaginazione) e setta "Formato dell'ora" al tipo "<script type="text/javascript">
		<!--
		var currentTime = new Date();
		var month = currentTime.getMonth() + 1;
		var day = currentTime.getDate();
		var year = currentTime.getFullYear();
		var hours = currentTime.getHours()
		var minutes = currentTime.getMinutes()
		if (day < 10) day = "0" + day;
		if (month < 10) month = "0" + month;
		if (hours < 10) hours = "0" + hours;
		if (minutes < 10) minutes = "0" + minutes;
		document.write(day + "/" + month + "/" + year + " " + hours + ":" + minutes);
		//-->
		</script>".
		</p><p>
		<label for="import-url"><?php _e('URL del blog: ' ); ?></label>http://
		<input type="text" id="import-url" name="import-url" size="20" />.splinder.com
		</p><p>
		<label for="start-month"><?php _e('Da:' ); ?></label>
		<select id="start-month" name="start-month">
			<?php
			for ($i = 1; $i <= 12; $i++) {
				echo "<option value=\"$i\"";
				if ($i == 1) echo ' selected="selected"';
				echo ">".$this->month_tostring($i)."</option>";
			}
			?>
		</select>
		<input type="text" id="start-year" name="start-year" value="2001" size="5" />
		</p><p>
		<label for="end-month"><?php _e('A:' ); ?></label>
		<select id="end-month" name="end-month">
			<?php
			for ($i = 1; $i <= 12; $i++) {
				echo "<option value=\"$i\"";
				if ($i == $this->today['mon']) echo ' selected="selected"';
				echo ">".$this->month_tostring($i)."</option>";
			}
			?>
		</select>
		<input type="text" id="end-year" name="end-year" value="<?php echo $this->today['year']; ?>" size="5" />
		</p><p>
		<label for="tagsorcats"><?php _e('Categorie Splinder:' ); ?></label><br />
		<input type="radio" id="tagsorcats" name="tagsorcats" value="tags" checked="checked" />Importa come tags di Wordpress<br />
		<input type="radio" id="tagsorcats" name="tagsorcats" value="cats" />Importa come categorie di Wordpress
		</p><p>
		<label for="uncategorized"><?php _e('Post senza categoria:' ); ?></label><br />
		<input type="radio" id="uncategorized" name="uncategorized" value="default" checked="checked" />Assegna alla mia categoria di default di WordPress<br />
		<input type="radio" id="uncategorized" name="uncategorized" value="other" />Assegna alla categoria...
		<input type="text" id="other-category" name="other-category" size="20" />
		</p><p>
		<input type="checkbox" id="verbose-posts" name="verbose-posts" value="posts" checked="checked" />Elenca tutti i post importati<br />
		<input type="checkbox" id="verbose-comments" name="verbose-comments" value="comments" />Elenca tutti i commenti importati
		</p>
		<p class="submit">
		<input type="submit" class="button" value="<?php esc_attr_e( 'Avvia importazione' ); ?>" />
		</p>
		</form>
		<?php

	}

	function dispatch() {
		if (empty ($_GET['step']))
			$step = 0;
		else
			$step = (int) $_GET['step'];

		$this->header();
		switch ($step) {
			case 0 :
				$this->greet();
				break;
			case 1 :
				check_admin_referer('import-splinder-form');
				$this->import("http://" . $_POST['import-url'] . ".splinder.com", $_POST['start-month'], $_POST['start-year'], $_POST['end-month'], $_POST['end-year']);
				break;
		}
		$this->footer();
	}

	function WP_Import() {
		// Nothing.
	}

}

/**
 * Register WordPress Importer
 *
 * @since unknown
 * @var Splinder_Import
 * @name $splinder_import
 */

$splinder_import = new Splinder_Import();

register_importer($splinder_import->importer_code, $splinder_import->importer_name, __('Importa un blog da Splinder','splinder-importer'), array ($splinder_import, 'dispatch'));

}
?>
